package com.lwby.marketing.att.dyvideo.handle;

import com.lwby.marketing.att.AttributionStatus;
import com.lwby.marketing.att.dyvideo.DyVideoUniversalProcess;
import com.lwby.marketing.att.novel.AttributionType;
import com.lwby.marketing.flow.NodeSwitchFlow;
import com.lwby.marketing.vo.AppChannelVO;
import com.lwby.marketing.vo.StoryNovelAction;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.concurrent.ThreadLocalRandom;

/**
 * @author songxinyu
 * @version PaySpduFlow.java, v 0.1 2024年03月04日 18:46 songxinyu Exp $
 */
@Slf4j
@Component("dyvideo_dedu")
public class DyvideoDeduFlow extends NodeSwitchFlow<StoryNovelAction> {

    private static final Logger DYVIDEO_SYS_LOG   = LoggerFactory.getLogger("dyvideo.sys");

    @Resource
    DyVideoUniversalProcess up;

    @Override
    public boolean checked(StoryNovelAction action) {
        return action.getType() == 3;
    }

    @Override
    public void process(StoryNovelAction action) {
        int platformId = action.getPlatformId();
        Long channelId = action.getChannelId();
        AppChannelVO appChannel = up.getAppChannelByPlatformAndChannel(platformId,channelId);
        Integer sprDedu = appChannel.getSprDedu();

        //等于空 或 100直接回传
        if (sprDedu == null || sprDedu == 100) {
            return;
        }

        //总数
        String channelTotal =  up.getTotalCountKey(AttributionType.CHANNEL, action.getPlatformId(), action.getChannelId(), sprDedu, action.getCurrentDateStr());
        //回传
        String channelCallback = up.getCallbackCountKey(AttributionType.CHANNEL, action.getPlatformId(), action.getChannelId(), sprDedu, action.getCurrentDateStr());

        long channelTotalCount = up.incrby(channelTotal, 0, 60 * 60 * 24);
        long channelCallbackCount = up.incrby(channelCallback, 0, 60 * 60 * 24);

        up.incrby(channelTotal, 1);


        if (isCallback(channelTotalCount, channelCallbackCount, sprDedu)) {
            //回传，回传个数 + 1
            up.incrby(channelCallback, 1);
            DYVIDEO_SYS_LOG.info(
                    "ChannelAttributionFlow.process0.deduction doing dynamic, platformId = {}, channel = {}, planId={}, sprDedu = {}, channelTotalCount = {}, channelCallbackCount = {}, v = {}",
                    action.getPlatformId(), action.getChannelId(),action.getPlanId(), sprDedu, channelTotalCount, channelCallbackCount, 1);
        } else {
            up.notifyResult(action,"ocpc_behavior_test", AttributionStatus.NORMAL_DEDUCTION_CALLBACK);
            up.set(up.getFirstCheckerKey(action),60 * 60 * 24,"1");
            DYVIDEO_SYS_LOG.info(
                    "ChannelAttributionFlow.process0.deduction doing dynamic, platformId = {}, channel = {}, planId={}, sprDedu = {}, channelTotalCount = {}, channelCallbackCount = {}, v = {}",
                    action.getPlatformId(), action.getChannelId(),action.getPlanId(), sprDedu, channelTotalCount, channelCallbackCount, 0);
        }

    }

    private boolean isCallback(long channelTotalCount, long channelCallbackCount, Integer sprDedu) {
        if (channelTotalCount == 0) {
            //首次随机
            return ThreadLocalRandom.current().nextInt(1, 3) == 1;
        }
        //计算回传率
        BigDecimal divide = new BigDecimal(channelCallbackCount).divide(new BigDecimal(channelTotalCount), 4, RoundingMode.HALF_UP);
        //比较回传率和扣量比例，决定是否回传
        double percent = (double) Math.round(sprDedu * 100 / 100.0) / 100;
        return divide.compareTo(BigDecimal.valueOf(percent).setScale(4, RoundingMode.HALF_UP)) != 1;
    }

}
